@@ -18,14 +18,15 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =end
 
 # Feed2Imap version
-F2I_VERSION = '0.5'
+F2I_VERSION = '0.7'
 
 require 'feed2imap/config'
 require 'feed2imap/cache'
-require 'feed2imap/channel'
 require 'feed2imap/httpfetcher'
 require 'logger'
 require 'thread'
+require 'feedparser'
+require 'feed2imap/itemtomail'
 
 class Feed2Imap
   def Feed2Imap.version
@@ -46,6 +47,10 @@ class Feed2Imap
       @logger.fatal("Configuration file #{configfile} not found.")
       exit(1)
     end
+    if (File::stat(configfile).mode & 044) != 0
+      @logger.warn("Configuration file is readable by other users. It " +
+        "probably contains your password.")
+    end
     begin
       File::open(configfile) { 
         |f| @config = F2IConfig::new(f)
@@ -57,10 +62,20 @@ class Feed2Imap
     # init cache
     @logger.info('Initializing cache')
     @cache = ItemCache::new
+    if not File::exist?(@config.cache + '.lock')
+      f = File::new(@config.cache + '.lock', 'w')
+      f.close
+    end
+    if File::new(@config.cache + '.lock').flock(File::LOCK_EX | File::LOCK_NB) == false
+      @logger.fatal("Another instance of feed2imap is already locking the cache file")
+      exit(1)
+    end
     if not File::exist?(@config.cache) 
       @logger.warn("Cache file #{@config.cache} not found, using a new one")
     else
-      File::open(@config.cache) { |f| @cache.load(f) }
+      File::open(@config.cache) do |f|
+        @cache.load(f)
+      end
     end
     # connecting all IMAP accounts
     @logger.info('Connecting to IMAP accounts')
@@ -84,28 +99,37 @@ class Feed2Imap
     end
     # for each feed, fetch, upload to IMAP and cache
     @logger.info("Fetching feeds")
-    loggermon = Mutex::new
     ths = []
+    mutex = Mutex::new
     @config.feeds.each do |f|
-      ths << Thread::new do
+      ths << Thread::new(f) do |feed|
         begin
-          lastcheck = @cache.get_last_check(f.name) 
-          if f.needfetch(lastcheck)
-            f.body = HTTPFetcher::fetch(f.url, @cache.get_last_check(f.name))
-            @cache.set_last_check(f.name, Time::now)
+          mutex.lock
+          lastcheck = @cache.get_last_check(feed.name) 
+          if feed.needfetch(lastcheck)
+            mutex.unlock
+            s = HTTPFetcher::fetch(feed.url, @cache.get_last_check(feed.name))
+            mutex.lock
+            feed.body = s
+            @cache.set_last_check(feed.name, Time::now)
           end
+          mutex.unlock
           # dump if requested
-          if @config.dumpdir and f.body
-            fname = @config.dumpdir + '/' + f.name + '-' + Time::now.xmlschema
-            File::open(fname, 'w') { |file| file.puts f.body }
+          if @config.dumpdir
+            mutex.synchronize do
+              if feed.body
+                fname = @config.dumpdir + '/' + feed.name + '-' + Time::now.xmlschema
+                File::open(fname, 'w') { |file| file.puts feed.body }
+              end
+            end
           end
         rescue Timeout::Error
-          loggermon.synchronize do
-            @logger.fatal("Timeout::Error while fetching #{f.url}: #{$!}")
+          mutex.synchronize do
+            @logger.fatal("Timeout::Error while fetching #{feed.url}: #{$!}")
           end
         rescue
-          loggermon.synchronize do
-            @logger.fatal("Error while fetching #{f.url}: #{$!}")
+          mutex.synchronize do
+            @logger.fatal("Error while fetching #{feed.url}: #{$!}")
           end
         end
       end
@@ -115,13 +139,13 @@ class Feed2Imap
     @config.feeds.each do |f|
       next if f.body.nil? # means 304
       begin
-        channel = Channel::new(f.body)
+        feed = FeedParser::Feed::new(f.body)
       rescue Exception => e
         @logger.fatal("Error while parsing #{f.name}: #{e}")
         next
       end
       begin
-        newitems, updateditems = @cache.get_new_items(f.name, channel.items)
+        newitems, updateditems = @cache.get_new_items(f.name, feed.items)
       rescue
         @logger.fatal("Exception caught when selecting new items for #{f.name}: #{$!}")
         puts $!.backtrace
@@ -130,8 +154,15 @@ class Feed2Imap
       @logger.info("#{f.name}: #{newitems.length} new items, #{updateditems.length} updated items.") if newitems.length > 0 or updateditems.length > 0
       begin
         if !cacherebuild
-          updateditems.each { |i| f.imapaccount.updatemail(f.folder, i.to_mail(f.name), i.cacheditem.index) }
-          newitems.each { |i| f.imapaccount.putmail(f.folder, i.to_mail(f.name)) }
+          updateditems.each do |i|
+            email = item_to_mail(i, i.cacheditem.index, true, f.name)
+            f.imapaccount.updatemail(f.folder, email,
+                                     i.cacheditem.index, i.date || Time::new)
+          end
+          newitems.each do |i|
+            email = item_to_mail(i, i.cacheditem.index, false, f.name)
+            f.imapaccount.putmail(f.folder, email, i.date || Time::new)
+          end
         end
       rescue
         @logger.fatal("Exception caught while uploading mail to #{f.folder}: #{$!}")
